@@ -1,125 +1,9 @@
-#' Setup a modeling procedure
-#'
-#' A modeling procedure is an object containing all information necessary to
-#' carry out and evaluate the performance of a predictive modeling task with
-#' \code{\link{fit}}, \code{\link{tune}}, or \code{\link{evaluate}}.
-#' To use an out-of-the box algorithm with default values, only the
-#' \code{method} argument needs to be set. See \code{\link{emil}} for a
-#' list of available methods. To deviate from the defaults, e.g. by tuning
-#' parameters or using a custom function for model fitting, set the appropriate
-#' parameters as described below.
-#' For a guide on how to implement a custom method see the documentaion page
-#' \code{\link{extension}}.
-#' 
-#' @param method The name of the modeling method. Only needed to identify
-#'   plug-in functions, i.e. if you supply them yourself there is no need to
-#'   set \code{method}.
-#' @param parameter A list of model parameters. These will be fed to the fitting
-#'   function after the dataset (\code{x} and \code{y} parameters). To tune a
-#'   parameter, supply the candidate values in a vector or list.
-#' 
-#'   When tuning more than one parameter, all combinations of parameter values
-#'   will be tested, if the elements of \code{parameter} are named. To manually
-#'   specify which parameter value combinations to try, leave the the elements
-#'   unnamed (see example 3 and 4).
-#'   
-#'   Parameters that should have vectors or lists as values, e.g. \code{trControl}
-#'   when using
-#'   \code{\link{fit_caret}} to train caret models, must be wrapped in an
-#'   additional list. That is, to set a parameter value to a list, but not tune it,
-#'   make it a list of length 1 containing the list to be used (see example 6).
-#' @param fit_fun The function to be used for model fitting.
-#' @param predict_fun The function to be used for model prediction.
-#' @param importance_fun The function to be used for calculating or extracting 
-#'   feature importances. See \code{\link{get_importance}} for details.
-#' @param error_fun Performance measure used to evaluate procedures
-#'   and to tune parameters. See \code{\link{error_fun}} for details.
-#' @return An object of class \code{modeling_procedure}.
-#' @example examples/modeling_procedure.r
-#' @seealso \code{\link{emil}}, \code{\link{evaluate}},
-#'   \code{\link{fit}}, \code{\link{tune}},
-#'   \code{\link[=predict.model]{predict}}, \code{\link{get_importance}}
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @export
-modeling_procedure <- function(method, parameter=list(), error_fun=NULL, fit_fun, predict_fun, importance_fun){
-    if(any(sapply(list(NA, FALSE), identical, parameter))){
-        warning("`parameter` must be supplied as a list. Assuming you really want `list()` i.e. to not set any parameters.")
-        parameter <- list()
-    }
-    if(!is.null(names(parameter))){
-        parameter <- apply(do.call(expand.grid, lapply(parameter, seq_along)), 1, function(i){
-            Map("[[", parameter, i)
-        })
-    }
-    procedure <- structure(class = "modeling_procedure", .Data = list(
-        method = if(missing(method)) NULL else method,
-        parameter = if(length(parameter) == 0) list() else if(length(parameter) == 1) parameter[[1]] else NULL,
-        tuning = if(length(parameter) < 2) NULL else list(parameter = parameter, error = NULL),
-        fit_fun =
-            if(missing(fit_fun)){
-                tryCatch(get(sprintf("fit_%s", method)), error=function(err) err)
-            } else fit_fun,
-        predict_fun =
-            if(missing(predict_fun)){
-                tryCatch(get(sprintf("predict_%s", method)), error=function(err) err)
-            } else predict_fun,
-        importance_fun =
-            if(missing(importance_fun)){
-                tryCatch(get(sprintf("importance_%s", method)), error=function(err) err)
-            } else importance_fun,
-        error_fun = error_fun
-    ))
-    if(!is.function(procedure$fit_fun))
-        stop("No fitting function found.")
-    procedure
-}
-
-
-#' Print method for modeling procedure
-#' 
-#' @method print modeling_procedure
-#' @param x modeling procedure.
-#' @param ... Ignored (kept for S3 consistency).
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @noRd
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @export
-print.modeling_procedure <- function(x, ...){
-    yn <- function(x){
-        if(is.null(x) || inherits(x, "error")){
-            "no"
-        } else {
-            if(isdebugged(x)) "yes (debug)" else "yes"
-        }
-    }
-    cat(sprintf(
-"%s modeling procedure.
-
-   model fitting function:       %s
-   prediction function:          %s
-   feature importance function:  %s
-   individual error function:    %s
-   
-   number of parameter sets to tune over: %i
-   tuned: %s\n",
-        if(is.null(x$method)) "Custom" else paste0("`", x$method, "`"),
-        yn(x$fit_fun), yn(x$predict_fun), yn(x$importance_fun), yn(x$error_fun),
-        if(is.null(x$tuning$parameter)) 1 else length(x$tuning$parameter),
-        if(is_tunable(x)) if(is_tuned(x)) "yes" else "no" else "not needed"
-    ))
-}
-
-
-#' Perform modeling
+#' Evaluate a modeling procedure
 #' 
 #' This function performs the important task of evaluating the performance of
 #' a modeling procedure with resampling, including tuning and pre-processing
 #' to not bias the results by information leakage.
 #'
-#' This function is the core of the framework, carrying out most of the work.
-#' It fits and evaluates models according to a resampling scheme, and extracts
-#' feature importance scores.
-#' 
 #' @param procedure Modeling procedure, or list of modeling procedures, as
 #'   produced by \code{\link{modeling_procedure}}.
 #' @param x Dataset, observations as rows and descriptors as columns.
@@ -146,8 +30,7 @@ print.modeling_procedure <- function(x, ...){
 #' @param .return_error If \code{FALSE} the entire modeling is aborted upon an
 #'   error. If \code{TRUE} the modeling of the particular fold is aborted and
 #'   the error message is returned instead of its results.
-#' @param .verbose Whether to print an activity log. Set to \code{-1} to
-#'   also suppress output generated from the procedure's functions.
+#' @param .verbose Whether to print an activity log.
 #' @return A list tree where the top level corresponds to folds (in case of
 #'   multiple folds), the next level corresponds to the modeling procedures
 #'   (in case of multiple procedures), and the final level is specified by the
@@ -166,22 +49,22 @@ print.modeling_procedure <- function(x, ...){
 #' @seealso \code{\link{emil}}, \code{\link{modeling_procedure}}
 #' @author Christofer \enc{Bäcklin}{Backlin}
 #' @export
-evaluate <- function(procedure, x, y,
-    resample=emil::resample("crossvalidation", y, nfold=2, nreplicate=2), pre_process=pre_split,
-    .save=c(model=TRUE, prediction=TRUE, importance=FALSE),
+evaluate <- function(procedure, x, y, resample, pre_process=pre_split,
+    .save=c(model=TRUE, prediction=TRUE, error=TRUE, importance=FALSE),
     .cores=1, .checkpoint_dir=NULL, .return_error=.cores > 1,
-    .verbose=TRUE){
+    .verbose=getOption("emil_verbose", TRUE)){
 
-    log_message(.verbose, "Evaluating modeling performance")
+    log_message(.verbose, "Evaluating modeling performance...")
 
-    if(inherits(procedure, "modeling_procedure")){
-        multi.procedure <- FALSE
-        procedure <- listify(procedure)
-    } else {
-        multi.procedure <- TRUE
-    }
-    debug.flags <- get_debug_flag(procedure)
-    if(is.logical(resample)){
+    procedure <- multify(procedure)
+    
+    # Get the real response vector since we will need it to prepare the analysis
+    y_original <- y
+    y <- get_response(x, y)
+
+    if(missing(resample))
+        resample <- emil::resample("crossvalidation", y, nfold=2, nrepeat=2)
+    if(inherits(resample, "fold")){
         multi.fold <- FALSE
         resample <- data.frame(resample)
         names(resample) <- attr(resample[[1]], "fold.name")
@@ -194,7 +77,7 @@ evaluate <- function(procedure, x, y,
         resample[make.na,] <- NA
     }
 
-    .save.default <- c(model=FALSE, prediction=TRUE, importance=FALSE)
+    .save.default <- c(model=FALSE, prediction=TRUE, error=TRUE, importance=FALSE)
     for(s in names(.save)){
         .save.default[pmatch(s, names(.save.default))] <- .save[s]
     }
@@ -234,7 +117,6 @@ evaluate <- function(procedure, x, y,
             }
         }
     }
-    procedure <- set_debug_flag(procedure, debug.flags)
 
 #------------------------------------------------------------------------------o
 #   Set up parallelization, error handling and checkpointing
@@ -289,8 +171,8 @@ evaluate <- function(procedure, x, y,
         }
 
         # Print status message
-        fold.message <- if(inherits(resample, "crossvalidation")){
-            sub("^fold(\\d+).(\\d+)$", "Replicate \\1, fold \\2", fold.name)
+        fold.message <- if(inherits(fold, "crossvalidation")){
+            sub("^rep(\\d+)fold(\\d+)$", "Repeat \\1, fold \\2", fold.name)
         } else if(inherits(resample, "holdout")){
             sub("^fold(\\d+)$", "Fold \\1", fold.name)
         } else {
@@ -313,7 +195,7 @@ evaluate <- function(procedure, x, y,
         if(any(do.tuning)){
             # Tune before extracting the training and testing sets (below) to
             # not produce any unnecessary in-memory copies of the dataset.
-            tune.subset <- subresample(fold, y)
+            tune.subset <- subresample(fold = fold, y = y)
             fold.procedure <- tune(procedure, x, y, resample=tune.subset,
                 pre_process=pre_process, .save=NULL, .verbose=indent(.verbose, 2))
         } else {
@@ -321,7 +203,11 @@ evaluate <- function(procedure, x, y,
         }
         log_message(indent(.verbose, 2), "Extracting fitting and testing datasets.")
         if(is.function(pre_process)){
-            sets <- pre_process(x, y, fold)
+            if(".verbose" %in% names(formals(pre_process))){
+                sets <- pre_process(x, y_original, fold, .verbose = indent(.verbose, 2))
+            } else {
+                sets <- pre_process(x, y_original, fold)
+            }
         } else if(is.list(pre_process)){
             sets <- pre_process[[1]](x, y, fold)
             if(length(pre_process) > 1) for(i in 2:length(pre_process)){
@@ -336,10 +222,12 @@ evaluate <- function(procedure, x, y,
             # Rather than doing all models at once we do one at a time in case
             # they require a lot of memory. The [[-workaround is to keep the
             # procedure names printing in the correct way.
-            model <- fit(fold.procedure[i], sets$fit$x, sets$fit$y,
+            model <- fit(fold.procedure[i], x=sets$fit$x, y=sets$fit$y,
                          .verbose=indent(.verbose, 2))[[1]]
-            prediction <- predict(model, sets$test$x, .verbose=indent(.verbose, 1))
-            list(error = fold.procedure[[i]]$error_fun(sets$test$y, prediction),
+            prediction <- predict(object=model, x=sets$test$x, .verbose=indent(.verbose, 1))
+            list(error = if(.save["error"]){
+                    fold.procedure[[i]]$error_fun(sets$test$y, prediction)
+                 } else NULL,
                  model = if(.save["model"]) model else NULL,
                  prediction = if(.save["prediction"]) prediction else NULL,
                  importance = if(.save["importance"]) get_importance(model)
@@ -364,14 +252,12 @@ evaluate <- function(procedure, x, y,
                 t2 <- t1 + difftime(Sys.time(), t1, units="sec")*ncol(resample)
                 fmt <- if(difftime(t2, t1, units="days") < 1){
                     "%H:%M"
-                } else if(difftime(t2, t1, units="days") < 2 &&
-                          as.integer(strftime(t2, "%prediction")) -
-                          as.integer(strftime(t1, "%prediction")) == 1){
+                } else if(difftime(t2, t1, units="days") < 2){
                     "%H:%M tomorrow"
                 } else if(difftime(t2, t1, units="days") < 365){
-                    "%H:%M, %b %prediction"
+                    "%H:%M, %b %d"
                 } else {
-                    "%H:%M, %b %prediction, %Y"
+                    "%H:%M, %b %d, %Y"
                 }
                 log_message(indent(.verbose, 1),
                           "Estimated completion time %sis %s.",
@@ -382,7 +268,7 @@ evaluate <- function(procedure, x, y,
         }
 
         # Save checkpoint file
-        if(!multi.procedure) res <- res[[1]]
+        if(!attr(procedure, "multiple")) res <- res[[1]]
         if(!is.null(checkpoint.file)){
             save(res, file=checkpoint.file)
         }
@@ -393,20 +279,6 @@ evaluate <- function(procedure, x, y,
 }
 
 
-#' Extractor function for modeling result
-#' 
-#' As opposed to the standard extractor function, this will keep the class.
-#' 
-#' @param x modeling result object, as produced by \code{\link{evaluate}}.
-#' @param ... Sent to \code{\link{`[`}}.
-#' @noRd
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @export
-`[.modeling_result` <- function(x, ...){
-    structure(unclass(x)[...], class="modeling_result")
-}
-
-
 #' Fit a model
 #'
 #' Fits a model according to a modeling procedure. If the procedure contains
@@ -422,20 +294,15 @@ evaluate <- function(procedure, x, y,
 #'   suppress all messages.
 #' @return A list of fitted models.
 #' @examples
-#' procedure <- modeling_procedure("lda")
-#' mod <- fit(procedure, x=iris[-5], y=iris$Species)
+#' mod <- fit("lda", x=iris[-5], y=iris$Species)
 #' @seealso \code{\link{emil}}, \code{\link{modeling_procedure}},
 #'   \code{\link{evaluate}}, \code{\link{tune}},
 #'   \code{\link[=predict.model]{predict}}, \code{\link{get_importance}}
 #' @author Christofer \enc{Bäcklin}{Backlin}
 #' @export
-fit <- function(procedure, x, y, ..., .verbose=TRUE){
-    if(inherits(procedure, "modeling_procedure")){
-        multi.procedure <- FALSE
-        procedure <- listify(procedure)
-    } else {
-        multi.procedure <- TRUE
-    }
+fit <- function(procedure, x, y, ..., .verbose=getOption("emil_verbose", FALSE)){
+    #log_message(.verbose, "Fitting models...")
+    procedure <- multify(procedure)
 
     missing.fun <- unlist(lapply(procedure, function(p)
         if(!is.function(p$fit_fun)) sprintf("fit_%s", p$method) else NULL))
@@ -447,15 +314,19 @@ fit <- function(procedure, x, y, ..., .verbose=TRUE){
     need.tuning <- !sapply(procedure, is_tuned)
     if(any(need.tuning)){
         procedure[need.tuning] <- tune(procedure[need.tuning], x=x, y=y, ...,
-            .verbose=.verbose)
+            .verbose=indent(.verbose, 1))
     }
     res <- Map(function(p, mn){
         log_message(.verbose, "Fitting %s", mn)
-        structure(list(fit = do.call(function(...) p$fit_fun(x, y, ...), p$parameter),
-                       procedure = p),
-                  class="model")
-    }, procedure, name_procedure(procedure))
-    if(multi.procedure) res else res[[1]]
+        if(".verbose" %in% names(formals(p$fit_fun)))
+            p$parameter$.verbose <- indent(.verbose, 1)
+        model <- do.call(function(...) p$fit_fun(x=x, y=y, ...), p$parameter)
+        structure(list(model = model, procedure = p), class="model")
+    }, procedure, names(procedure))
+    # Restore debug flags (for some reason Map removes them)
+    for(i in seq_along(procedure))
+        debug_flag(res[[i]]$procedure) <- debug_flag(procedure[[i]])
+    if(attr(procedure, "multiple")) res else res[[1]]
 }
 
 
@@ -480,21 +351,14 @@ fit <- function(procedure, x, y, ..., .verbose=TRUE){
 #'   \code{\link[=predict.model]{predict}}, \code{\link{get_importance}}
 #' @author Christofer \enc{Bäcklin}{Backlin}
 #' @export
-tune <- function(procedure, ..., .verbose=FALSE){
-    log_message(.verbose, "Tuning parameters")
-    if(inherits(procedure, "modeling_procedure")){
-        multi.procedure <- FALSE
-        procedure <- listify(procedure)
-    } else {
-        multi.procedure <- TRUE
-    }
-    debug.flags <- get_debug_flag(procedure)
+tune <- function(procedure, ..., .verbose=getOption("emil_verbose", FALSE)){
+    log_message(.verbose, "Tuning parameters...")
+    procedure <- multify(procedure)
+
     do.tuning <- sapply(procedure, is_tunable)
     discard.tuning <- do.tuning & sapply(procedure, is_tuned)
-    if(any(discard.tuning)){
-        for(i in which(discard.tuning))
-            procedure[[i]]$tuning <- NULL
-    }
+    for(i in which(discard.tuning))
+        procedure[[i]]$tuning <- NULL
     tune.procedure <- lapply(procedure[do.tuning], function(p){
         lapply(p$tuning$parameter, function(pp){
             p$parameter <- pp
@@ -502,42 +366,50 @@ tune <- function(procedure, ..., .verbose=FALSE){
             p
         })
     })
-    tune.name <- lapply(tune.procedure, name_procedure)
+    tune.name <- rep(names(tune.procedure), sapply(tune.procedure, length))
     tune.procedure <- unlist(tune.procedure, recursive=FALSE)
-    names(tune.procedure) <- unlist(tune.name)
-    tune.procedure <- set_debug_flag(tune.procedure, rep(debug.flags,
-        each=sapply(procedure, function(p) length(p$tuning$parameter) )))
+    names(tune.procedure) <- ave(tune.name, tune.name, FUN=function(x){
+        if(length(x) == 1) x else sprintf("%s (%i)", x, seq_along(x))
+    })
 
     tuning <- evaluate(tune.procedure, ..., .verbose=indent(.verbose, 1))
     procedure.id <- rep(which(do.tuning),
                    sapply(procedure[do.tuning], function(p) length(p$tuning$parameter)))
     for(i in which(do.tuning)){
         procedure[[i]]$tuning$error <-
-            sapply(tuning, function(tun) unlist(subtree(tun, procedure.id == i, "error")))
-        rownames(procedure[[i]]$tuning$error) <- NULL
-        mean.err <- apply(procedure[[i]]$tuning$error, 1, mean)
-        best.parameter <- which(mean.err == min(mean.err))
-        if(length(best.parameter) > 1) best.parameter <- sample(best.parameter, 1)
-        procedure[[i]]$parameter <- procedure[[i]]$tuning$parameter[[best.parameter]]
+            select(tuning, fold=TRUE, method = procedure.id == i, error = "error") %>%
+            mutate_(parameter_set = "as.integer(method)") %>%
+            select_("-method")
+        best_parameter <- procedure[[i]]$tuning$error %>%
+            group_by_("parameter_set") %>%
+            summarize_(mean_error = "mean(error)") %>%
+            filter_("mean_error == min(mean_error)") %>%
+            sample_n(1)
+        procedure[[i]]$parameter <-
+            procedure[[i]]$tuning$parameter[[best_parameter$parameter_set]]
         procedure[[i]]$tuning$result <- lapply(tuning, "[", procedure.id == i)
     }
-    procedure <- set_debug_flag(procedure, debug.flags)
-    if(multi.procedure) procedure else procedure[[1]]
+    if(attr(procedure, "multiple")) procedure else procedure[[1]]
 }
 #' @rdname tune
 #' @return Logical indicating if the procedure(s) are tuned.
 #' @export
-is_tuned <- function(procedure)
+is_tuned <- function(procedure){
+    stopifnot(inherits(procedure, "modeling_procedure"))
     !is_tunable(procedure) || !is.null(procedure$parameter)
+}
 #' @rdname tune
 #' @return Logical indicating if the has tunable parameters.
 #' @export
-is_tunable <- function(procedure)
+is_tunable <- function(procedure){
+    stopifnot(inherits(procedure, "modeling_procedure"))
     !is.null(procedure$tuning)
+}
 #' @rdname tune
 #' @return A list of untuned modeling procedures.
 #' @export
 detune <- function(procedure){
+    stopifnot(inherits(procedure, "modeling_procedure"))
     debug.flags <- sapply(procedure, function(p) is.function(p) && isdebugged(p))
     procedure$tuning <- NULL
     lapply(procedure[debug.flags], debug)
@@ -551,12 +423,10 @@ detune <- function(procedure){
 #' @param object Fitted model.
 #' @param x Data set with observations whose response is to be predicted.
 #' @param ... Sent to the procedure's prediction function.
-#' @param .verbose Whether to print an activity log. Set to \code{-1} to
-#'   suppress all messages.
+#' @param .verbose Whether to print an activity log.
 #' @return See the documentation of procedure's method.
 #' @examples
-#' procedure <- modeling_procedure("lda")
-#' mod <- fit(procedure, x=iris[-5], y=iris$Species)
+#' mod <- fit("lda", x=iris[-5], y=iris$Species)
 #' prediction <- predict(mod, iris[-5])
 #' @author Christofer \enc{Bäcklin}{Backlin}
 #' @seealso \code{\link{emil}}, \code{\link{modeling_procedure}},
@@ -564,59 +434,42 @@ detune <- function(procedure){
 #'   \code{\link{get_importance}}
 #' @export
 predict.model <- function(object, x, ..., .verbose=FALSE){
-    if(.verbose < 0){
-        capture.output(suppressMessages(
-            res <- object$procedure$predict_fun(object = object$fit, x = x, ...)
-        ))
-        res
+    if(".verbose" %in% names(formals(object$procedure$predict_fun))){
+        p <- object$procedure$predict_fun(object = object$model, x = x, ..., .verbose=.verbose)
     } else {
-        object$procedure$predict_fun(object = object$fit, x = x, ...)
+        p <- object$procedure$predict_fun(object = object$model, x = x, ...)
     }
+    structure(p, class="prediction")
 }
 
-#' Preserve debugging flags when manipulating lists
-#'
-#' \emph{In R version >= 3.1.0 these functions are no longer needed due to
-#' changed behavior of the debugging facilities.}
-#'
-#' These functions are designed to copy and reset debugging flags when
-#' manipulating modeling procedures or lists of such. See the code of
-#' \code{\link{tune}}, e.g. by calling \code{page(tune)} for an example
-#' of how to work with them.
-#'
-#' By default, when performing an operation on a list, in whole or in part, the
-#' whole list will be overwritten by a new list containing the results of the
-#' operation. This will discard debugging flags set by \code{\link{debug}},
-#' which is not desirable when working with lists containing functions.
-#'
-#' @param procedure A modeling procedure to be wrapped in a one-element-list.
-#' @param procedure.list A list of modeling procedures.
-#' @param debug.flags A list of debugging flags, as produced by
-#'   \code{\link{get_debug_flag}}
-#' @return \code{list(x)} with debug flags maintained.
+
+#' Extract the response from a data set
+#' 
+#' @param x Data set features.
+#' @param y Response vector or any other type of objects that describe how to
+#'   extract the response vector from \code{x}.
+#' @return A response vector.
+#' @examples
+#' identical(iris$Species, get_response(iris, "Species"))
+#' identical(iris$Sepal.Length, get_response(iris, Sepal.Length ~ .))
 #' @author Christofer \enc{Bäcklin}{Backlin}
-#' @noRd
-listify <- function(procedure){
-    debug.flags <- sapply(procedure, function(p) is.function(p) && isdebugged(p))
-    procedure <- list(procedure)
-    lapply(procedure[[1]][debug.flags], debug)
-    procedure
+#' @export
+get_response <- function(x, y){
+    UseMethod("get_response", y)
 }
-# @rdname listify
-#' @noRd
-get_debug_flag <- function(procedure.list){
-    lapply(procedure.list, sapply, function(p) is.function(p) && isdebugged(p))
+#' @method get_response default
+#' @export
+get_response.default <- function(x, y){
+    y
 }
-# @rdname listify
-#' @noRd
-set_debug_flag <- function(procedure.list, debug.flags){
-    if(as.integer(R.Version()$major) < 3 || as.numeric(R.Version()$minor) < 1){
-        for(i in seq_along(debug.flags)){
-            for(j in seq_along(debug.flags[[i]])){
-                if(debug.flags[[i]][j]) debug(procedure.list[[i]][[j]])
-            }
-        }
-    }
-    procedure.list
+#' @method get_response default
+#' @export
+get_response.character <- function(x, y){
+    x[,y]
+}
+#' @method get_response default
+#' @export
+get_response.formula <- function(x, y){
+    model.response(model.frame(y, x))
 }
 
